#include "Behaviors/StateMachine.h"
#include "DualCoding/DualCoding.h"

using namespace DualCoding;

template <class T>
Point radialSearch(const Point &origin, const Point &startPt, const Point &endPt, Sketch<T> sketch) {
    
	ShapeSpace space = sketch.rootGetSpace().getDualSpace();
	T* pixelData = sketch->getRawPixels();
	LineData connection = LineData(space, startPt, endPt);
	std::vector<float> eq = connection.lineEquation_abc();
	
	for (size_t i = 0; i < connection.getLength(); i++) {
		LineData testLine = LineData(space, startPt, Point(eq[0]*i + connection.end1Pt().coordX() , eq[1]*i + connection.end1Pt().coordY()));
		//iterate along the line, return point of first intersection
		std::vector<float> currLineEq = testLine.lineEquation_abc();
		
		for(size_t j = 0; j < testLine.getLength(); j++) {
			if(pixelData[(int)(currLineEq[1] * j * sketch->getHeight() + currLineEq[0] * j)])
				return Point(currLineEq[1] * j * sketch->getHeight(), currLineEq[0] * j);
		}
	}
	
	//if we got here then we didn't find anything
	return NULL;
}


template <class T>
ShapeRoot radialSearch(ShapeRoot &origin, const Point &startPt, const Point &endPt, const UnaryShapePred<T> &pred) {
    
    ShapeSpace space = origin.getSpace();
    NEW_SHAPEROOTVEC(matches, subset(space, pred()));
	//find the shapes which are inside of the triangle defined by the radial search
    std::vector<Point> pointData = std::vector<Point>(3);
    pointData[0] = origin->getCentroid();
    pointData[1] = startPt;
    pointData[2] = endPt;
    PolygonData searchArea = PolygonData(space, pointData,true, true, true);
    std::vector<ShapeRoot> containedShapes;
    
    //iterate through the shapevec
    std::vector<ShapeRoot >::iterator ln_it;
    for (ln_it = matches.begin(); ln_it != matches.end(); ln_it++ ) {
        ShapeRoot &currShape = *ln_it;
        
		//should get extreme edges as well
        if (searchArea.isInside(currShape->getCentroid())) {
            //add to containedshapes
            containedShapes ;
        }
        
    };
    
	//find the shape which is not to the left of anything
    
    for (ln_it = matches.begin(); ln_it != matches.end(); ln_it++ ) {
        ShapeRoot &currShape = *ln_it;
        
        
        std::vector<ShapeRoot >::iterator ln_it2;
        for (ln_it2 = matches.begin(); ln_it2 != matches.end(); ln_it2++ ) {
            ShapeRoot &currShape2 = *ln_it2;
            
            
            if (currShape->getCentroid().isRightOf(currShape2->getCentroid())) {
                break;
            }
        };
        //if we get here, then we have a shape which isn't to the right of anything, and therefore is the rightmost shape
        return currShape;
        
    };
    
    //something went wrong if we got here
	return NULL;
	
}


template <class T, class U>
Point contourSearch(const Point &origin, const Point &startPt, Sketch<T> sketch, const UnaryShapePred<U> &pred) {
	ShapeSpace space = sketch->getSpace().getDualSpace();
	T* pixelData = sketch->getRawPixels();
	NEW_SHAPEROOTVEC(matches, subset(space, pred()));
	
	//make a queue of pixels to visit, with origin at its head
	std::queue<Point> toVisit = queue<Point>();
	toVisit.push(origin);
	//define a set of already visited pixels
    std::set<Point> visited = set<Point>();
	//expand outward from the pixel of origin
	while(toVisit.size() > 0)
	{
		Point testPoint = toVisit.pop();
        
		for(size_t i = 0; i < 3; i++) {
            
            for(size_t j = 0; j < 3; j++) {
                
                Point curr = Point(testPoint.coordX() - 1 + i, testPoint.coordY() - 1 + j);
                //because insert returns false if the set already has testpoint in it, we can test to see if we've already visited it using this method
                if(visited.insert(testPoint) && isContourPoint(curr, pixelData, sketch->getWidth(), sketch->getHeight(), sketch->getNumPixels()))
                    toVisit.push(curr);
            }
        }
        
        //check if it is in  the bounds of any of the shapes in matches
        std::vector<ShapeRoot >::iterator ln_it;
        for (ln_it = matches.begin(); ln_it != matches.end(); ln_it++ ) {
            ShapeRoot &currShape = *ln_it;
            if(currShape->isInside(testPoint)) {
                //if it is then return that point
                return testPoint;
            }
        };
        
	}
	
	//if we got here then there isn't anything that matches the predicate on the contour
	return NULL;
}

/*
 tests if a pixel is part of the contour. A contour-pixel has at least one pixel which is empty next to it -- this method tests all adjacent pixels to see if they are empty
 */

template <class T>
bool isContourPoint(const Point &testPoint, const T* const pixelData, const size_t sketchWidth, const size_t sketchLength, const size_t sketchSize) {
	for(size_t i = 0; i < 3; i++) {
		
		for(size_t j = 0; j < 3; j++) {
            
			Point curr = Point(testPoint.coordX() - 1 + i, testPoint.coordY() - 1 + j);
			//if this point is out of bounds then go on to the next one
			if(curr.coordX() > sketchWidth || curr.coordX() < 0 || curr.coordY() < 0 || curr.coordY() > sketchLength) {
				continue;
			}
			
			if(pixelData[sketchWidth * testPoint.coordY() + testPoint.coordX()]) {
				return true;
			}
		}
	}
	
	return false;
}




